function PSD_PhaseNoise_Dynamical

w0 = 190;  % pJ
U0 = w0/2;   % pJ
tau0 = 235*1e-3/1.763;  % ps

A0 = sqrt(U0/tau0);
phi = 1/2*A0^2;
taus = 1;
T = 50*tau0;
N = 512;
a = hme_eigen();
a.setup(T,N);

us = @(t) A0*sech(t/tau0);

TS.mode = 'tw2tau';
TS.N = 512;
TS.tw2tau = 50;
TS.switch_FourierFD = 'off';
fprintf('g0 = %.5g, b2 = %.5g, gam = %.5g, \n', a.g0, a.b2, a.gam);

a.g0 = 7.74;

tic;
a.newton_solver(us(a.t),taus,phi,TS);

% figure(10); hold on;
% plot(a.t,abs(us(a.t)),'r--');
% plot(a.t,abs(a.NTout.Uout),'b.');

if a.NTout.res > 1e-5
    error('No Convergence!!!');
end
    
%% Eigen-analysis
switch_FourierFD_Eigen = 'on';
switch_LeftEigen = 'on';
[V_left] = a.CompleteEigen(switch_FourierFD_Eigen,switch_LeftEigen);

%% Measurement and Projections;
w0 = norm(a.NTout.Uout)^2*a.dt*1e-12;  % unit: SI
h_measure = [a.t.*a.NTout.Uout;conj(a.t.*a.NTout.Uout)]*1e-12/w0;  % UNIT: SI, For timing jitter

ER = [a.ev(1:a.N,:)+1i*a.ev((1+a.N):end,:); a.ev(1:a.N,:)-1i*a.ev((1+a.N):end,:)];  % Say, unit: 1e6
EL = conj([V_left(1:a.N,:)-1i*V_left((1+a.N):end,:); V_left(1:a.N,:)+1i*V_left((1+a.N):end,:)]/2);   % So, unit: 1e6
Lambda = a.ew;
hx = ER'*h_measure*a.dt;  % unit: 1e6*1e-12 = 1e-6 

% figure(11);
% plot(real(a.ew),imag(a.ew),'.');

%% Sub-setting the matrices: only keep elements in h_x that are significant
ind_big = find(abs(hx)/max(abs(hx)) > 1e-5);
ind = max(ind_big):length(Lambda);
EL(:,ind) = [];
Lambda(ind) = [];
hx(ind) = [];

%% The diffusion coefficient "D" of the Gaussian white noise
wavelength0 = 1564e-9;
nu0 = 3e8/wavelength0;  % unit: SI
h_planck = 6.62607004e-34; % SI
TR = 1/300e6;
S_auto = a.NTout.gsat*h_planck*nu0*TR;  % unit: Watt * s

H = conj(hx)*hx.'; % unit: 1e-12 SI
D = (EL'*EL)*a.dt*S_auto;  %  unit: 1e12*1e-12 = SI

Nm = length(Lambda);
Omega = Lambda*ones(1,Nm);
M = H.*D;  % unit: 1e-36
Sh_f = @(f) sum(sum(M./((Omega-2i*pi*f).*(Omega'+2i*pi*f))));

%% Calculating the PSD using the dynamical method
faxis  = [0:0.05:50]*1e6; % the frequency axis
f_comp = faxis*TR;  % normalized frequency we use in computation

parfor ifreq = 1:length(faxis)
    PSD_h(ifreq) = Sh_f(f_comp(ifreq))*1e-12;  % unit: SI
end

%% Timing Jitter Calculation
PSD_dt = (2*pi*f_comp).^2.*PSD_h;  % unit: SI
toc

% figure(2); hold on;
% plot((faxis),10*log10(2*PSD_dt),'--');
% axis tight;
% xlim([0,35]*1e6);
% grid on;

figure(1); hold on;
PSD_phase = PSD_dt./(f_comp*TR).^2;
plot((faxis),10*log10(2*PSD_phase),'--');
axis tight;
xlim([0,35]*1e6);
grid on;

%% Haus Mecozzi Method
if 1
    Parameters_SESAM300_paper2;
    wavelength0 = 1564e-9; % unit: SI
    nu0 = 3e8/wavelength0;  % unit: SI
    TR = 1/300e6; % unit: SI
    h_planck = 6.62607004e-34; % SI
    S_auto = a.NTout.gsat*h_planck*nu0*TR;  % unit: Watt * s
    
    w0 = norm(a.NTout.Uout)^2*a.dt*1e-12;  % unit: SI
    tau0 = PlsWidth_FWHM(a.t,a.NTout.Uout)/1.763*1e-12;   % unit: SI
    Omega = Omega*1e12;
    beta2 = beta2*1e-24; % unit: SI
    
    var_f = 2/(3*w0*tau0^2)*S_auto;  % unit: SI
    var_t = (pi*tau0)^2/(6*w0)*S_auto; % unit: SI
    
    %% Decay Coefficients
    rf = 2*a.NTout.gsat/3/(Omega*sqrt(2)*tau0)^2; % unitless
    rt = beta2;  % unit: SI
    
    Stf_analytical = rt^2*var_f./(rf^2+(2*pi*f_comp).^2) + var_t;
%     figure(2);
%     plot(faxis,10*log10(2*Stf_analytical),'r-');

    figure(1);
    hold on;
    Sphif_analytical = Stf_analytical./(TR*f_comp).^2;
    plot(faxis,10*log10(2*Sphif_analytical),'r-');
end

legend({'Dynamical','Haus-Mecozzi'});
end
